Package: Rcsdp
Version: 0.1.57.6
Title: R Interface to the CSDP Semidefinite Programming Library
Authors@R: c(
    person("Hector", "Corrada Bravo", email="hcorrada@gmail.com", role=c("aut", "cre")),
    person("Florian", "Schwendinger", email="FlorianSchwendinger@gmx.at", role=c("ctb")),
    person("Brian", "Borchers", role="aut"),
    person("Don", "van den Bergh", role="ctb")
  )
Description: R interface to the CSDP semidefinite programming library. Installs version 6.1.1 of CSDP from the COIN-OR website if required. An existing installation of CSDP may be used by passing the proper configure arguments to the installation command. See the INSTALL file for further details.
LazyLoad: yes
Imports: methods
Enhances: Matrix
License: CPL-1.0
URL: https://github.com/coin-or/Csdp/
RoxygenNote: 7.0.2
BugReports: https://github.com/hcorrada/rcsdp/issues
NeedsCompilation: yes
Packaged: 2025-06-23 10:22:38 UTC; hornik
Author: Hector Corrada Bravo [aut, cre],
  Florian Schwendinger [ctb],
  Brian Borchers [aut],
  Don van den Bergh [ctb]
Maintainer: Hector Corrada Bravo <hcorrada@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-23 10:24:35 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 14:09:25 UTC; unix
Archs: Rcsdp.so.dSYM

Package: Rcsdp
Version: 0.1.57.6
Title: R Interface to the CSDP Semidefinite Programming Library
Authors@R: c(
    person("Hector", "Corrada Bravo", email="hcorrada@gmail.com", role=c("aut", "cre")),
    person("Florian", "Schwendinger", email="FlorianSchwendinger@gmx.at", role=c("ctb")),
    person("Brian", "Borchers", role="aut"),
    person("Don", "van den Bergh", role="ctb")
  )
Description: R interface to the CSDP semidefinite programming library. Installs version 6.1.1 of CSDP from the COIN-OR website if required. An existing installation of CSDP may be used by passing the proper configure arguments to the installation command. See the INSTALL file for further details.
LazyLoad: yes
Imports: methods
Enhances: Matrix
License: CPL-1.0
URL: https://github.com/coin-or/Csdp/
RoxygenNote: 7.0.2
BugReports: https://github.com/hcorrada/rcsdp/issues
NeedsCompilation: yes
Packaged: 2025-06-23 10:22:38 UTC; hornik
Author: Hector Corrada Bravo [aut, cre],
  Florian Schwendinger [ctb],
  Brian Borchers [aut],
  Don van den Bergh [ctb]
Maintainer: Hector Corrada Bravo <hcorrada@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-23 10:24:35 UTC

useDynLib(Rcsdp, .registration = TRUE, .fixes = "C_")

importFrom("methods", "as")
importFrom("stats", "rnorm")

export("csdp",
       "csdp_minimal",
       "csdp.control",
       "readsdpa",
       "writesdpa",
       "readsdpa.sol",
       "writesdpa.sol",
       "simple_triplet_sym_matrix",
       ".simple_triplet_zero_sym_matrix",
       ".simple_triplet_diag_sym_matrix",
       ".simple_triplet_random_sym_matrix")

S3method("as.simple_triplet_sym_matrix","simple_triplet_sym_matrix")
S3method("as.simple_triplet_sym_matrix","matrix")
S3method("as.matrix","simple_triplet_sym_matrix")
S3method("as.vector","simple_triplet_sym_matrix")
S3method("dim","simple_triplet_sym_matrix")
S3method("all.equal","simple_triplet_sym_matrix")

S3method("as.simple_triplet_sym_matrix","dgeMatrix")
S3method("as.simple_triplet_sym_matrix","dsyMatrix")
S3method("as.simple_triplet_sym_matrix","dpoMatrix")
S3method("as.simple_triplet_sym_matrix","dgTMatrix")
S3method("as.simple_triplet_sym_matrix","dsCMatrix")
S3method("as.simple_triplet_sym_matrix","dsTMatrix")
S3method("as.simple_triplet_sym_matrix","ddiMatrix")

# Rcsdp

<!-- badges: start -->
<!-- badges: end -->

This package is an R interface to the CSDP semidefinite programming library by Brian Borchers (https://github.com/coin-or/Csdp/)

## Installation

You can install the released version of Rcsdp from [CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("Rcsdp")
```

# Rcsdp.R
# Interface to the CSDP semidefinite programming library by Brian Borchers
# https://github.com/coin-or/Csdp/
#
# Created: 22 February 2008
# Author: Hector Corrada Bravo

csdp <- function(C,A,b,K,control=csdp.control()) {
  prob.info <- get.prob.info(K,length(b))
  validate.data(C,A,b,prob.info)
  prob.data <- prepare.data(C,A,b,prob.info)
  write.control.file(control)
  
  ret <- .Call(C_csdp,
               as.integer(sum(prob.info$block.sizes)),
               as.integer(prob.info$nconstraints),
               as.integer(prob.info$nblocks),
               as.integer(c(0,prob.info$block.types)),
               as.integer(c(0,prob.info$block.sizes)),
               prob.data$C,
               prob.data$A,
               prob.data$b,
               PACKAGE="Rcsdp")

  unlink("param.csdp")
  ret[1:3] <- get.solution(ret[[1]],ret[[2]],ret[[3]],prob.info)
  structure(ret,names=c("X","Z","y","pobj","dobj","status"))
}

get.solution <- function(X,Z,y,prob.info)
  {
    list(X=blkmatrix_csdp2R(X,prob.info),
         Z=blkmatrix_csdp2R(Z,prob.info),
         y=vector_csdp2R(y))
      }

prepare.data <- function(C,A,b,prob.info)
  {
    list(C=blkmatrix_R2csdp(C,prob.info),
         A=constraints_R2csdp(A,prob.info),
         b=as.double(vector_R2csdp(b)))
  }

get.prob.info <- function(K,m) {
 if (!all.equal(names(K),c("type","size")))
   stop("Invalid cone specification 'K': elements must be 'type' and 'size'")

 if (!all(K$type %in% c("s","l")))
   stop("Invalid cone specification 'K': types must be 's' or 'l'")

 if (length(K$type) != length(K$size))
   stop("Invalid conse specification 'K': type and size elements must be of the same length")

 
 block.types <- ifelse(K$type == "s",1,2)
 nblocks <- length(K$type)
 
 block.sizes <- K$size
 nconstraints <- m;

 ret <- list(nblocks=nblocks,
             nconstraints=nconstraints,
             block.types=block.types,
             block.sizes=block.sizes)
 return(ret)
}

validate.data <- function(C,A,b,prob.info) {
  nblocks <- prob.info$nblocks
  nconstraints <- prob.info$nconstraints
  block.types <- prob.info$block.types
  block.sizes <- prob.info$block.sizes
  
  # Validate number of blocks in C
  if (length(C) != nblocks)
    stop("Number of blocks in C disagrees with K")

  # Validate number of constraint matrices in A
  if (length(A) != nconstraints)
    stop("Number of constraint matrices in A disagrees with b")

  # Validate each block of C
  for (j in 1:nblocks) {
    if ((block.types[j] == 1) && (block.sizes[j] != nrow(C[[j]]) || block.sizes[j] != ncol(C[[j]])))
      stop("Size of block ",j," in C does not agree with K")
    if ((block.types[j] == 2) && (block.sizes[j] != length(C[[j]])))
      stop("Size of block ",j," in C does not agree with K")
  }

  # Validate constraint matrices
  for (i in 1:nconstraints) {
    Ai <- A[[i]];
    if (length(Ai) != nblocks)
        stop("Number of blocks in constraint matrix ",i," in A disagrees with K")
             
    for (j in 1:nblocks) {
      if ((block.types[j] == 1) && (block.sizes[j] != nrow(Ai[[j]]) || block.sizes[j] != ncol(Ai[[j]])))
        stop("Size of block ",j," in constraint matrix ",i," in A does not agree with K")
      if ((block.types[j] == 2) && (block.sizes[j] != length(Ai[[j]])))
        stop("Size of block ",j," in constraint ",i," of A does not agree with K")
    }
  }
}

csdp.control <- function(axtol=1e-8,
                          atytol=1e-8,
                          objtol=1e-8,
                          pinftol=1e8,
                          dinftol=1e8,
                          maxiter=100,
                          minstepfrac=0.90,
                          maxstepfrac=0.97,
                          minstepp=1e-8,
                          minstepd=1e-8,
                          usexzgap=1,
                          tweakgap=0,
                          affine=0,
                          printlevel=1,
                          perturbobj=1,
                          fastmode=0)
  {
    list(axtol=axtol,
         atytol=atytol,
         objtol=objtol,
         pinftol=pinftol,
         dinftol=dinftol,
         maxiter=maxiter,
         minstepfrac=minstepfrac,
         maxstepfrac=maxstepfrac,
         minstepp=minstepp,
         minstepd=minstepd,
         usexzgap=usexzgap,
         tweakgap=tweakgap,
         affine=affine,
         printlevel=printlevel,
         perturbobj=perturbobj,
         fastmode=fastmode)
  }

write.control.file <- function(control)
  {
    fileptr <- file("param.csdp","w")
    for (i in 1:length(control))
      cat(names(control)[i],"=",control[[i]],"\n",sep="",file=fileptr)
    close(fileptr)
  }

csdp_minimal <- function(sum.block.sizes, nconstraints, nblocks, block.types, block.sizes, C, A, b) {
  return(.Call(
    C_csdp,
    as.integer(sum.block.sizes),
    as.integer(nconstraints),
    as.integer(nblocks),
    as.integer(block.types),
    as.integer(block.sizes),
    C,
    A,
    b,
    PACKAGE="Rcsdp"
  ))
}

## A simple class for sparse symmetric (triplet) matrices.
##
## Mostly copied from sparse.R in Kurt Hornik's relations package

simple_triplet_sym_matrix <-
function(i, j, v, n = max(c(i,j)),check.ind=FALSE)
{
  if (check.ind & any(i<j)) {
      stop("Index arguments 'i' and 'j' do not point to the lower triangle. Swapping indices")
    }
  structure(list(i = i, j = j, v = v, n = n),
            class = "simple_triplet_sym_matrix")
}

as.simple_triplet_sym_matrix <-
function(x,...)
    UseMethod("as.simple_triplet_sym_matrix")

as.simple_triplet_sym_matrix.simple_triplet_sym_matrix <- function(x,...) x

as.simple_triplet_sym_matrix.matrix <-
function(x,check.sym=FALSE, ...)
{
    if(prod(dim(x)) == 0L)
        return(simple_triplet_sym_matrix(integer(), integer(), c(x), n=nrow(x)))

    if (nrow(x) != ncol(x))
      stop("Argument 'x' must be a square matrix")

    if (check.sym && (sum(abs(x-t(x))/2) != 0))
        stop("Argument 'x' must be a symmetric matrix")
    
    ind <- which(x != vector(typeof(x), 1L), arr.ind = TRUE)
    if (length(ind) == 0)
      return(.simple_triplet_zero_sym_matrix(nrow(x)))
             
    ind <- ind[ind[,1L] >= ind[,2L],,drop=FALSE]
    simple_triplet_sym_matrix(ind[, 1L], ind[, 2L], x[ind],
                          n = nrow(x))
}

as.matrix.simple_triplet_sym_matrix <-
function(x, ...)
{
    n <- x$n
    y <- matrix(vector(typeof(x$v), n * n), n, n)
    ind <- cbind(x$i, x$j)
    y[ind] <- x$v
    offdiag <- x$i != x$j
    y[ind[offdiag,c(2L,1L),drop=FALSE]] <- x$v[offdiag]
    y
}

as.vector.simple_triplet_sym_matrix <-
  function(x, ...)
  {
    as.vector(as.matrix(x,...))
  }

as.double.simple_triplet_sym_matrix <-
  function(x,...)
  {
    as.double(as.vector(x,...))
  }

dim.simple_triplet_sym_matrix <-
function(x)
    c(x$n, x$n)

all.equal.simple_triplet_sym_matrix <-
  function(target,current,...)
  {
    if(!inherits(current,"simple_triplet_sym_matrix"))
      stop("Argument 'current' is not of class 'simple_triplet_sym_matrix'")
    
    if(!inherits(target,"simple_triplet_sym_matrix"))
       stop("Argument 'target' is not of class 'simple_triplet_sym_matrix'")

    cur.ord <- order(current$j,current$i)
    current$i <- current$i[cur.ord]
    current$j <- current$j[cur.ord]
    current$v <- current$v[cur.ord]
    
    targ.ord <- order(target$j,target$i)
    target$i <- target$i[targ.ord]
    target$j <- target$j[targ.ord]
    target$v <- target$v[targ.ord]

    all.equal.list(current,target,...)
  }

## Utitilies for creating special simple triplet matrices:

.simple_triplet_zero_sym_matrix <-
function(n, mode = "double")
    simple_triplet_sym_matrix(integer(), integer(), vector(mode, 0L),n)

.simple_triplet_diag_sym_matrix <-
function(x, n)
{
    x <- rep(x, length.out = n)
    i <- seq_len(n)
    simple_triplet_sym_matrix(i, i, x, n)
}

.simple_triplet_random_sym_matrix <-
function(n,occ=.1,nnz=occ*n*(n+1)/2,rfun=rnorm,seed=NULL,...)
  {
    if(!missing(seed) & !is.null(seed))
      set.seed(seed)

    # sample indices in the lower triangle
    iind <- sample(n*(n+1)/2,nnz)
    ind <- 1:(n*(n+1)/2) + sapply(rep(1:n,times=n:1), function(x) x*(x-1)/2)
    ind <- ind[iind]
    simple_triplet_sym_matrix(i=((ind-1) %% n) + 1,
                          j=((ind-1) %/% n) + 1,
                          v=rfun(nnz,...),
                          n=n)
  }

# Convert data to/from csdp format
# to R objects (matrix or simple_triplet_sym_matrix)

# vectors
vector_R2csdp <- function(x)
  {
    c(0,x);
  }

vector_csdp2R <- function(x)
  {
    x[-1];
  }

blkmatrix_R2csdp <- function(X,prob.info)
  {
    do.one.block <- function(blocknum)
      {
        cur.block <- X[[blocknum]];
        cur.type <- prob.info$block.types[blocknum];
        cur.size <- prob.info$block.sizes[blocknum];
        
        if (cur.type == 1) {
          data <- as.double(cur.block);
        }
        else {
          data <- vector_R2csdp(cur.block)
        }
        structure(list(blocksize=as.integer(cur.size),
             blockcategory=as.integer(cur.type),
             data=as.double(data)),class="csdpBlkMat")
      }
    
    nblocks <- prob.info$nblocks;
    list(nblocks=nblocks,
         blocks=lapply(seq_along(X),do.one.block))
  }

all.equal.csdpBlkMat <-
  function(target, current, ...) {
    all.equal.list(target,current,...)
  }

blkmatrix_csdp2R <- function(X,prob.info)
  {
    do.one.block <- function(blocknum)
      {
        cur.block <- X$blocks[[blocknum]];
        cur.type <- prob.info$block.types[blocknum];
        cur.size <- prob.info$block.sizes[blocknum];
        
        names(cur.block) <- c("blocksize","blockcategory","data");
        if (cur.type == 1)
          matrix(cur.block$data,cur.size,cur.size)
        else
          vector_csdp2R(cur.block$data);
      }
    names(X) <- c("nblocks","blocks");
    lapply(seq_along(X$blocks),do.one.block);
  }

constraints_R2csdp <- function(A,prob.info)
  {
    nblocks <- prob.info$nblocks;

    do.one.constraint <- function(constraintnum)
      {
        Ai = A[[constraintnum]]
        ret <- vector("list",nblocks)

        k <- 0;
        for (j in 1:nblocks) {
          blocknum <- j;
          Aij <- Ai[[blocknum]];
          
          cur.type <- prob.info$block.types[blocknum];
          cur.size <- prob.info$block.sizes[blocknum];
          
          if (cur.type == 1) {
            tmp <- as.simple_triplet_sym_matrix(Aij);
            if (length(tmp$v) == 0)
              next
            
            iindices <- vector_R2csdp(tmp$i);
            jindices <- vector_R2csdp(tmp$j);
            entries <- vector_R2csdp(tmp$v);
          }
          else {
            nnz <- which(Aij != 0);
            if (length(nnz) == 0)
              next
            
            iindices <- vector_R2csdp(nnz);
            jindices <- vector_R2csdp(nnz);
            entries <- vector_R2csdp(Aij[nnz]);
          }
          k <- k+1
          ret[[k]] <- structure(list(iindices=as.integer(iindices),
                           jindices=as.integer(jindices),
                           entries=as.double(entries),
                           blocknum=as.integer(blocknum),
                           blocksize=as.integer(cur.size),
                           constraintnum=as.integer(constraintnum),
                           numentries=as.integer(length(entries)-1)),class="csdpConstrMat")

        }
        ret[1:k]
      }
    lapply(seq_along(A),do.one.constraint);
  }

all.equal.csdpConstrMat <-
  function(target, current, ...) {
    res <- all.equal.list(target[c("blocknum","blocksize","constraintnum","numentries")],
                          current[c("blocknum","blocksize","constraintnum","numentries")],...)
    if (is.logical(res)) {
      if (!res) return(FALSE)
      return(all.equal(simple_triplet_sym_matrix(i=target$iindices[-1],
                                        j=target$jindices[-1],
                                        v=target$entries[-1]),
                       simple_triplet_sym_matrix(i=target$iindices[-1],
                                        j=target$jindices[-1],
                                        v=target$entries[-1])))
    }
    return(res)
  }
        

    

constraints_csdp2R <- function(A,prob.info)
  {
    nblocks <- prob.info$nblocks
    do.one.constraint <- function(constraintnum)
      {
        Ai <- A[[constraintnum]]
        ret <- mapply(FUN=function(type, size) {
          if (type == 1) {
            .simple_triplet_zero_sym_matrix(size)
          } else {
            rep(0, size)
          }}, prob.info$block.types, prob.info$block.sizes, SIMPLIFY=FALSE)
        
        for (j in 1:length(Ai)) {
          Aij <- Ai[[j]];
          names(Aij) <- c("iindices","jindices","entries",
                          "blocknum","blocksize","constraintnum","numentries");
          if (Aij$constraintnum != constraintnum)
            stop("Constraint number in csdp constraint matrix is not right")

          cur.type <- prob.info$block.types[Aij$blocknum];
          cur.size <- prob.info$block.sizes[Aij$blocknum];

          if (cur.type == 1) {
            cur.iindices <- vector_csdp2R(Aij$iindices);
            cur.jindices <- vector_csdp2R(Aij$jindices);

            # simple_triplet_sym_matrix uses the lower triangle
            if (any(cur.iindices < cur.jindices)) {
              tmp <- cur.iindices
              cur.iindices <- cur.jindices
              cur.jindices <- tmp
            }
            ret[[Aij$blocknum]] <- simple_triplet_sym_matrix(i=cur.iindices,
                                                             j=cur.jindices,
                                                             v=vector_csdp2R(Aij$entries),
                                                             n=cur.size,
                                                             check.ind=TRUE);
            
          }
          else {
            tmp <- rep(0.0,cur.size);
            tmp[vector_csdp2R(Aij$iindices)] <- vector_csdp2R(Aij$entries)
            ret[[Aij$blocknum]] <- tmp;
          }
        }
        ret
      }
    lapply(seq_along(A),do.one.constraint);
  }

# functions to support sparse matrices from the Matrix package
.not.dsTMatrix <- function(x,cl)
  {
    if(!inherits(x,cl))
      stop("Argument 'x' must be of class ", cl)
    as.simple_triplet_sym_matrix(as(x,"dsTMatrix"))
  }

as.simple_triplet_sym_matrix.dgeMatrix <- function(x, ...) .not.dsTMatrix(x,"dgeMatrix")
as.simple_triplet_sym_matrix.dsyMatrix <- function(x, ...) .not.dsTMatrix(x,"dsyMatrix")
as.simple_triplet_sym_matrix.dpoMatrix <- function(x, ...) .not.dsTMatrix(x,"dpoMatrix")
as.simple_triplet_sym_matrix.dgTMatrix <- function(x, ...) .not.dsTMatrix(x,"dgTMatrix")
as.simple_triplet_sym_matrix.dsCMatrix <- function(x, ...) .not.dsTMatrix(x,"dsCMatrix")
as.simple_triplet_sym_matrix.dsTMatrix <-
  function(x, ...)
  {
    if(!inherits(x,"dsTMatrix"))
      stop("Argument 'x' must be of class 'dsTMatrix'")

    # simple_triplet_sym_matrix store entries in the
    # lower triangle only
    if (x@uplo %in% c("u","U")) {
      x@uplo <- "l"
      tmp <- x@i
      x@i <- x@j
      x@j <- tmp
    }
    
    x <- as(x,"dsTMatrix")
    simple_triplet_sym_matrix(i=x@i+1,
                              j=x@j+1,
                              v=x@x,
                              n=nrow(x))
  }

#setAs("simple_triplet_sym_matrix","dsTMatrix",
#  function(x)
#  {
#    new("dgTMatrix",i=as.integer(x$i-1),j=as.integer(x$j-1),x=v,Dim=as.integer(c(x$n,x$n)),uplo="L")
#  })

as.simple_triplet_sym_matrix.ddiMatrix <- function(x, ...)
  {
    if(!inherits(x,"ddiMatrix"))
       stop("Argument 'x' must be of class 'ddiMatrix'")

    v <- if (x@diag=="U") rep(1,nrow(x)) else x@x
    simple_triplet_sym_matrix(i=1:nrow(x),
                              j=1:nrow(x),
                              v=v,
                              n=nrow(x))
  }

coerce_mat <- function(mat,cfun=as.simple_triplet_sym_matrix) if (is.null(dim(mat))) mat else cfun(mat)
coerce_blkmat <- function(Aij,cfun=as.simple_triplet_sym_matrix) lapply(Aij,coerce_mat,cfun=cfun)
coerce_const <- function(Ai,cfun=as.simple_triplet_sym_matrix) lapply(Ai,coerce_blkmat,cfun=cfun)

getdf_mat <- function(x)
  {
    if(!is.null(dim(x))) {
      x <- as.simple_triplet_sym_matrix(x)
      if (length(x$v) == 0)
        return(NULL)
      else
        return(cbind(x$j,x$i,x$v))
    }
    else {
      ind <- which(x != 0)
      if (length(ind) == 0)
        return(NULL)
      else
        return(cbind(ind,ind,x[ind]))
    }
  }

getdf_blkmat <- function(x)
  {
    do.call(rbind,lapply(1:length(x),function(j) {df <- getdf_mat(x[[j]]); if (is.null(df)) return(NULL) else return(cbind(j,df))}))
  }

getdf_const <- function(x)
  {
    do.call(rbind,lapply(1:length(x), function(i) {df <- getdf_blkmat(x[[i]]); if (is.null(df)) return(NULL) else return(cbind(i,df))}))
  }

readsdpa <- function(file="",verbose=FALSE)
  {
    if (file=="")
      stop("'file' argument must be a non-empty string")
    
    ret <- .Call(C_readsdpa,
                 as.character(file),
                 as.integer(verbose),
                 PACKAGE="Rcsdp")

    names(ret) <- c("C","A","b","K");
    names(ret$K) <- c("type","size");
    ret$K$type <- c("s","l")[vector_csdp2R(ret$K$type)];
    ret$K$size <- vector_csdp2R(ret$K$size);

    m <- length(ret$b)-1;
    prob.info <- get.prob.info(ret$K,m)
    ret$C <- blkmatrix_csdp2R(ret$C,prob.info);
    ret$A <- constraints_csdp2R(ret$A,prob.info);
    ret$b <- vector_csdp2R(ret$b);
    ret
  }

readsdpa.sol <- function(K,C,m,file="")
  {
    if (file=="")
      stop("'file' argument must be a non-empty string")

    prob.info <- get.prob.info(K,m);
    ret <- .Call(C_readsdpa_sol,
                 as.character(file),
                 as.integer(sum(prob.info$block.sizes)),
                 as.integer(m),
                 blkmatrix_R2csdp(C,prob.info),
                 PACKAGE="Rcsdp");
    names(ret) <- c("X","y","Z");
    ret$X <- blkmatrix_csdp2R(ret$X,prob.info);
    ret$y <- vector_csdp2R(ret$y);
    ret$Z <- blkmatrix_csdp2R(ret$Z,prob.info);
    ret
  }

writesdpa <- function(C,A,b,K,file="")
  {
    if (file=="")
      stop("'file' argument must be a non-empty string")

    prob.info <- get.prob.info(K,length(b));
    validate.data(C,A,b,prob.info)
    prob.data <- prepare.data(C,A,b,prob.info)
    .Call(C_writesdpa,
          as.character(file),
          as.integer(sum(prob.info$block.sizes)),
          as.integer(prob.info$nconstraints),
          as.integer(prob.info$nblocks),
          as.integer(c(0,prob.info$block.types)),
          as.integer(c(0,prob.info$block.sizes)),
          prob.data$C,
          prob.data$A,
          prob.data$b,
          PACKAGE="Rcsdp")
  }

writesdpa.sol <- function(X,Z,y,K,file="")
  {
    if (file=="")
      stop("'file' argument must be a non-empty string")

    prob.info <- get.prob.info(K,length(y));

    .Call(C_writesdpa_sol,
          as.character(file),
          as.integer(sum(prob.info$block.sizes)),
          as.integer(prob.info$nconstraints),
          blkmatrix_R2csdp(X,prob.info),
          vector_R2csdp(y),
          blkmatrix_R2csdp(Z,prob.info),
          PACKAGE="Rcsdp")
  }

\name{readsdpa}
\alias{readsdpa}
\alias{writesdpa}
\alias{readsdpa.sol}
\alias{writesdpa.sol}
\title{Reading and writing semidefinite programs for SDPA format files.}
\description{Functions to read and write semidefinite program data and
  solutions in SDPA format.}
\usage{
   readsdpa(file="",verbose=FALSE)
   writesdpa(C,A,b,K,file="")
   readsdpa.sol(K,C,m,file="")
   writesdpa.sol(X,Z,y,K,file="")
}
\arguments{
  \item{file}{The name of the file to read from or write to.}
  \item{C}{Block structured cost matrix}
  \item{A}{List of block structured constraint matrices}
  \item{b}{RHS vector}
  \item{K}{Cone specification, as used in \code{\link{csdp}}}
  \item{X}{Block structured primal optimal solution matrix}
  \item{Z}{Block structured dual optimal solution matrix}
  \item{y}{Dual optimal solution vector}
  \item{verbose}{Printout information as problem is read. Passed to CSDP's readsdpa function. Default \code{FALSE}}
  \item{m}{Number of constraints in problem.}
}
\details{
  Block structured matrices must be specified as described in
  \code{\link{csdp}}.
  Files read must be in SDPA format (see
  \url{http://euler.nmt.edu/~brian/sdplib/FORMAT}).
  However, these functions don't support comments or grouping characters
  (e.g. braces, parentheses) in the block sizes specification.
}
\value{
  Function \code{readsdpa} returns a list with elements \code{C,A,b,K}.
  Function \code{readsdpa.sol} returns a list with elements
  \code{X,Z,y}.
  All returned matrices are lists of objects of class \code{simple_triplet_sym_matrix}.
}
\references{
  \url{http://euler.nmt.edu/~brian/sdplib/FORMAT}
}
\author{Hector Corrada Bravo}
\seealso{\code{\link{csdp}}}
\examples{
#  TO DO
}

\name{csdp-sparse}
\alias{simple_triplet_sym_matrix-class}
\alias{simple_triplet_sym_matrix}
\alias{as.simple_triplet_sym_matrix.matrix}
\alias{as.matrix.simple_triplet_sym_matrix}
\alias{as.vector.simple_triplet_sym_matrix}
\alias{.simple_triplet_zero_sym_matrix}
\alias{.simple_triplet_diag_sym_matrix}
\alias{.simple_triplet_random_sym_matrix}
\title{Simple support for sparse matrices}
\description{Support for sparse matrices in package \code{Rcsdp}. The class
  \code{simple_triplet_sym_matrix} is defined to provide support
  for symmetric sparse matrices. It's definition is copied from the package \code{relations} by Kurt
  Hornik. Coercion functions from objects of class \code{matrix} and
  classes in the \code{Matrix} hierarchy are provided.}
\usage{
   simple_triplet_sym_matrix(i,j,v,n=max(c(i,j)),check.ind=FALSE)
   \method{as.simple_triplet_sym_matrix}{matrix}(x,check.sym=FALSE,...)
   \method{as.matrix}{simple_triplet_sym_matrix}(x,...)
   \method{as.vector}{simple_triplet_sym_matrix}(x,...)
   .simple_triplet_zero_sym_matrix(n,mode="double")
   .simple_triplet_diag_sym_matrix(x,n)
   .simple_triplet_random_sym_matrix(n,occ=.1,nnz=occ*n*(n+1)/2,rfun=rnorm,seed=NULL,...)
}
\arguments{
  \item{i}{Row indices of non-zero entries.}
  \item{j}{Column indices of non-zero entries.}
  \item{v}{Non-zero entries.}
  \item{n}{Size of matrix.}
  \item{check.ind}{Checks that arguments \code{i} and \code{j} indicate entries in the lower triangular part of the matrix. Default \code{FALSE}.}
  \item{check.sym}{Checks if matrix object is symmetric. Default \code{FALSE}.}
  \item{x}{Object of class \code{matrix} or \code{simple_triplet_sym_matrix}.}
  \item{mode}{Type of zero matrix to create. Default \code{double}.}
  \item{occ}{Ratio of occupancy of random sparse matrix. Default \code{.1}.}
  \item{nnz}{Number of non-zero entries in random sparse matrix. Default corresponds to \code{occ=.1}.}
  \item{rfun}{Function to generate random entries in sparse matrix. Default \code{rnorm}.}
  \item{seed}{Random number generator seed. Set by function \code{set.seed} before generating random sparse matrix. Default \code{NULL}.}
  \item{...}{Arguments passed on to casting functions.}
}
\details{
  TO DO
}
\value{
  TO DO
}
\references{
  TO DO
}
\author{Hector Corrada Bravo}
\seealso{\link{csdp}}
\keyword{utilities}
\examples{
#  TO DO
}

\name{csdp.control}
\alias{csdp.control}
\title{
Pass control parameters to csdp solver.
}
\description{
Utility function to pass control parameters to csdp solver.
}
\usage{
csdp.control(axtol = 1e-08, 
  atytol = 1e-08, 
  objtol = 1e-08, 
  pinftol = 1e+08, 
  dinftol = 1e+08, 
  maxiter = 100, 
  minstepfrac = 0.9, 
  maxstepfrac = 0.97, 
  minstepp = 1e-08, 
  minstepd = 1e-08, 
  usexzgap = 1, 
  tweakgap = 0, 
  affine = 0, 
  printlevel = 1, 
  perturbobj = 1, 
  fastmode = 0)
}
\arguments{
  \item{axtol}{Tolerance for primal feasibility.
}
  \item{atytol}{Tolerance for dual feasibility.
}
  \item{objtol}{
Tolerance for relative duality gap.
}
  \item{pinftol}{
Tolerance for primal infeasibility.
}
  \item{dinftol}{
Tolerance for dual infeasibility.
}
  \item{maxiter}{
Maximum number of iterations used.
}
  \item{minstepfrac}{
Minimum distance to edge of feasibility region for step.
}
  \item{maxstepfrac}{
Maximum distance to edge of feasibility region for step.
}
  \item{minstepp}{
Failure is declared if primal line search step size is shorter than this parameter.
}
  \item{minstepd}{
Failure is declared if dual line search step size is shorter that this parameter.
}
  \item{usexzgap}{
If 0, then use objective function duality gap.
}
  \item{tweakgap}{
If 1 (and \code{usexzgap}=0) then "fix" negative duality gaps.
}
  \item{affine}{
If 1, only use affine primal-dual steps and do not use barrier function.
} 
  \item{printlevel}{
If 0, no printing, 1 normal printing, higher values result in more debug printing.
}
  \item{perturbobj}{
Amount of objective permutation used.
}
  \item{fastmode}{
If 1, csdp will be faster but also less accurate.
}
}
\details{
Parameters are fully described in CSDP user guide. \url{https://github.com/coin-or/Csdp/}}
\value{
A list with values for all parameters. Any parameters not passed to function are set to default.
}
\references{
\url{https://github.com/coin-or/Csdp/}
}
\author{
Hector Corrada Bravo, CSDP by Brian Borchers
}
\examples{
params <- csdp.control(axtol=1e-6)
}

\name{csdp}
\alias{csdp}
\alias{csdp_minimal}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Solve semidefinite program with CSDP}
\description{
	Interface to CSDP semidefinite programming library. The general statement of the primal problem is
	\deqn{\max\, \mathrm{tr}(CX)}{max tr(CX)}
	\deqn{\mathrm{s.t.}\; A(X) = b}{s.t. A(X) = b}
	\deqn{X \succeq 0}{X >= 0}
	with
	\eqn{A(X)_i = \mathrm{tr}(A_iX)}{A(X)_i = tr(A_iX)}
	where \eqn{X \succeq 0}{X >= 0} means X is positive
	semidefinite, \eqn{C} and all \eqn{A_i} are symmetric matrices of the same
	size and \eqn{b} is a
	vector of length \eqn{m}. 
	
	The dual of the problem is
	\deqn{\min\, b'y}{min b'y}
	\deqn{\mathrm{s.t.}\; A'(y) - C = Z}{s.t. A'(y) - C = Z}
	\deqn{Z \succeq 0}{Z >= 0}

	where
	\eqn{A'(y) = \sum_{i=1}^m y_i A_i.}{A'(y) = \sum_{i=1}^m y_i A_i.}

	Matrices \eqn{C} and \eqn{A_i} are assumed to be block diagonal
	structured, and must be specified that way (see Details). 
}
\usage{
csdp(C, A, b, K,control=csdp.control())
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{C}{A list defining the block diagonal cost matrix \eqn{C}.}
  \item{A}{A list of length \eqn{m} containing block diagonal constraint matrices
    \eqn{A_i}. Each constraint matrix \eqn{A_i} is specified by a list
	of blocks as explained in the Details section.}
  \item{b}{A numeric vector of length \eqn{m} containing the right hand side of the constraints.}
  \item{K}{Describes the domain of each block of the sdp problem. It is a list with the following elements:
    \describe{
      \item{type:}{A character vector with entries \code{"s"} or
      \code{"l"} indicating the type of each block. If the \code{j}th entry
      is \code{"s"}, then the \code{j}th block is a positive semidefinite matrix.
      otherwise, it is a vector with non-negative entries.}
      \item{size:}{A vector of integers indicating the dimension of each
      block.}      
      }}
  \item{control}{Control parameters passed to csdp. See CSDP documentation.}
}
\details{
  All problem matrices are assumed to be of block diagonal structure,
  and must be specified as follows:
  \enumerate{
    \item If there are \code{nblocks} blocks specified by \code{K}, then
    the matrix must be a list with \code{nblocks} components.
    \item If \code{K$type == "s"} then the \code{j}th element of the list must define a symmetric
	matrix of size \code{K$size}. It can be an object of class
	\code{"matrix"}, \code{"simple_triplet_sym_matrix"}, or a valid
	class from the class hierarchy in the \code{"Matrix"} package.
   \item If \code{K$type == "l"} then the \code{j}th element of the list
	must be a numeric vector of length \code{K$size}.
  }
  
  This function checks that the blocks in arguments \code{C} and \code{A} agree with
	the sizes given in argument \code{K}. It also checks that the
	lengths of arguments \code{b} and \code{A}
	are equal. It does not check for symmetry in the problem data.
	
	\code{csdp_minimal} is a minimal wrapper to the C code underlying \code{csdp}. It assumes that the arguments 
	\code{sum.block.sizes}, \code{nconstraints}, \code{nblocks}, \code{block.types}, and \code{block.sizes}
	are provided as if they were created by \code{Rcsdp:::prob.info} and that the arguments \code{C}, \code{A}, and 
	\code{b} are provided as if they were created by \code{Rcsdp:::prepare.data}. This function may be useful when 
	calling the csdp functionality iteratively and most of the optimization details stays the same. For example, when the
	control file created by \code{Rcsdp:::write.control.file} stays the same across iterations, but it would be recreated
	on each iteration by \code{csdp}.
}
\value{
  \item{X}{Optimal primal solution \eqn{X}. A list containing blocks in the
	same structure as explained above. Each element is of class
	\code{"matrix"} or a numeric vector as appropriate.}
  \item{Z}{Optimal dual solution \eqn{Z}. A list containing blocks in the same
	structure as explained above. Each element is of class
	\code{"matrix"}
	or a numeric vector as appropriate.}
  \item{y}{Optimal dual solution \eqn{y}. A vector of the same length as
    argument \code{b}}
  \item{pobj}{Optimal primal objective value}
  \item{dobj}{Optimal dual objective value}
  \item{status}{Status of returned solution.
    \describe{
      \item{0:}{Success. Problem solved to full accuracy}
      \item{1:}{Success. Problem is primal infeasible}
      \item{2:}{Success. Problem is dual infeasible}
      \item{3:}{Partial Success. Solution found but full accuracy was
	not achieved}
      \item{4:}{Failure. Maximum number of iterations reached}
      \item{5:}{Failure. Stuck at edge of primal feasibility}
      \item{6:}{Failure. Stuch at edge of dual infeasibility}
      \item{7:}{Failure. Lack of progress}
      \item{8:}{Failure. \eqn{X} or \eqn{Z} (or Newton system \eqn{O}) is singular}
      \item{9:}{Failure. Detected NaN or Inf values}
    }
    }
}
\references{
  \itemize{
  \item \url{https://github.com/coin-or/Csdp/}
  \item Borchers, B.:\cr
      \emph{CSDP, A C Library for Semidefinite Programming} Optimization Methods and Software  11(1):613-623, 1999\cr
      \url{http://euler.nmt.edu/~brian/csdppaper.pdf}
    \item Lu, F., Lin, Y., and Wahba, G.:\cr
      \emph{Robust Manifold Unfolding with Kernel Regularization} TR
      1108, October, 2005. \cr
      \url{http://pages.stat.wisc.edu/~wahba/ftp1/tr1108rr.pdf}
}
}
\author{Hector Corrada Bravo. CSDP written by Brian Borchers. }
%\note{ ~~further notes~~ 
%
% ~Make other sections like Warning with \section{Warning }{....} ~
%}
%\seealso{ ~~objects to See Also as \code{\link{help}}, ~~~ }
\examples{
  C <- list(matrix(c(2,1,
                     1,2),2,2,byrow=TRUE),
            matrix(c(3,0,1,
                     0,2,0,
                     1,0,3),3,3,byrow=TRUE),
            c(0,0))
A <- list(list(matrix(c(3,1,
                        1,3),2,2,byrow=TRUE),
               matrix(0,3,3),
               c(1,0)),
          list(matrix(0,2,2),
               matrix(c(3,0,1,
                        0,4,0,
                        1,0,5),3,3,byrow=TRUE),
               c(0,1)))

  b <- c(1,2)
  K <- list(type=c("s","s","l"),size=c(2,3,2))
  csdp(C,A,b,K)

# Manifold Unrolling broken stick example
# using simple triplet symmetric matrices
X <- matrix(c(-1,-1,
              0,0,
              1,-1),nc=2,byrow=TRUE);
d <- as.vector(dist(X)^2);
d <- d[-2]

C <- list(.simple_triplet_diag_sym_matrix(1,3))
A <- list(list(simple_triplet_sym_matrix(i=c(1,2,2),j=c(1,1,2),v=c(1,-1,1),n=3)),
          list(simple_triplet_sym_matrix(i=c(2,3,3),j=c(2,2,3),v=c(1,-1,1),n=3)),
          list(matrix(1,3,3)))

K <- list(type="s",size=3)
csdp(C,A,c(d,0),K)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{optimize}
